#include "stdafx.h"
#include "bangs.h"
#include "main.h"
#include "group.h"
#include "utils.h"

typedef struct BangItem
{
	char szName[25];
	BangCommand *pCommand;
} BangItem;

void bangShow	(HWND caller, LPCSTR szArgs);
void bangHide	(HWND caller, LPCSTR szArgs);
void bangToggle	(HWND caller, LPCSTR szArgs);
void bangReset	(HWND caller, LPCSTR szArgs);
void bangSortByName	(HWND caller, LPCSTR szArgs);
void bangSortBySize	(HWND caller, LPCSTR szArgs);
void bangSortByType	(HWND caller, LPCSTR szArgs);
void bangSortByDate	(HWND caller, LPCSTR szArgs);
void bangSetTextColor (HWND caller, LPCSTR szArgs);
void bangSetTextBkColor (HWND caller, LPCSTR szArgs);
void bangSetBackground	(HWND caller, LPCSTR szArgs);
void bangSetBackgroundColor	(HWND caller, LPCSTR szArgs);
//void bangHook	(HWND caller, LPCSTR szArgs);
void bangToggleExplorerDesktop	(HWND caller, LPCSTR szArgs);

// it is possible to end the list with something like
// {NULL,NULL}, but for what?
#define BANGS_NUM		13

BangItem g_Bangs[BANGS_NUM] =
{
	{	"Show",		bangShow	},
	{	"Hide",		bangHide	},
	{	"Toggle",	bangToggle	},
	{	"Reset",	bangReset	},
	{	"SortByName",	bangSortByName	},
	{	"SortBySize",	bangSortBySize	},
	{	"SortByType",	bangSortByType	},
	{	"SortByDate",	bangSortByDate	},
	{	"SetTextColor",	bangSetTextColor},
	{	"SetTextBkColor",	bangSetTextBkColor},
	{	"SetBackground",	bangSetBackground},
	{	"SetBackgroundColor",	bangSetBackgroundColor},
	/*{	"Hook",		bangHook	}*/
	{	"ToggleExplorerDesktop",		bangToggleExplorerDesktop	}
};


void RegisterBangs()
{
	for (int i = 0; i < BANGS_NUM; i++)
	{
		char szBangName[MAX_BANGCOMMAND];;
		StringCchPrintf(szBangName, MAX_BANGCOMMAND, "!%s%s", g_szAppName, g_Bangs[i].szName);
		AddBangCommand(szBangName, g_Bangs[i].pCommand);
	}
}


void UnregisterBangs()
{
	for (int i = 0; i < BANGS_NUM; i++)
	{
		char szBangName[MAX_BANGCOMMAND];;
		StringCchPrintf(szBangName, MAX_BANGCOMMAND, "!%s%s", g_szAppName, g_Bangs[i].szName);
		RemoveBangCommand(szBangName);
	}
}


////////////////////////////////////////////////////////////////////////////
//
// Diagnostic message for !bangs
//
void BangFailed(const char *szBangName, const char *szGroup)
{
	utils::ErrorBoxPrint("Icon group with name \"%s\" not found\n!%s%s bang failed", szGroup, g_szAppName, szBangName);
}


void bangShow	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->Show();
	else
		BangFailed("Show", szArgs);
}

void bangHide	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->Hide();
	else
		BangFailed("Hide", szArgs);
}

void bangToggle	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->Toggle();
	else
		BangFailed("Toggle", szArgs);
}

void bangReset	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->Reset();
	else
		BangFailed("Reset", szArgs);
}

void bangSortByName	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->SortByName();
	else
		BangFailed("SortByName", szArgs);
}

void bangSortBySize	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->SortBySize();
	else
		BangFailed("SortBySize", szArgs);
}

void bangSortByType	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->SortByType();
	else
		BangFailed("SortByType", szArgs);
}

void bangSortByDate	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->SortByDate();
	else
		BangFailed("SortByDate", szArgs);
}

void bangSetTextColor	(HWND caller, LPCSTR szArgs)
{
	char token[MAX_LINE_LENGTH], szExtra[MAX_LINE_LENGTH];
	char* tokens[1];
	tokens[0] = token;
	LCTokenize(szArgs, tokens, 1, szExtra);
	CGroup *pGroup = GetGroupByName(tokens[0]);
	if (!pGroup)
	{
		BangFailed("SetTextColor", token);
		return;
	}
	pGroup->SetTextColor(utils::String2Color(szExtra));
}

void bangSetTextBkColor	(HWND caller, LPCSTR szArgs)
{
	char token[MAX_LINE_LENGTH], szExtra[MAX_LINE_LENGTH];
	char* tokens[1];
	tokens[0] = token;
	LCTokenize(szArgs, tokens, 1, szExtra);
	CGroup *pGroup = GetGroupByName(tokens[0]);
	if (!pGroup)
	{
		BangFailed("SetTextBkColor", token);
		return;
	}
	pGroup->SetTextBkColor(utils::String2Color(szExtra));
}

void bangSetBackground	(HWND caller, LPCSTR szArgs)
{
	char
		token1[MAX_LINE_LENGTH],
		token2[MAX_LINE_LENGTH],
		token3[MAX_LINE_LENGTH],
		token4[MAX_LINE_LENGTH],
		token5[MAX_LINE_LENGTH],
		szExtra[MAX_LINE_LENGTH];
	char* tokens[5] = {token1, token2, token3, token4, token5};
	LCTokenize(szArgs, tokens, 5, szExtra);
	CGroup *pGroup = GetGroupByName(token1);
	if (!pGroup)
	{
		BangFailed("SetBackground", token1);
		return;
	}
	pGroup->SetBackground(token2, atoi(token3), atoi(token4), utils::String2Bool(token5));
}

void bangSetBackgroundColor	(HWND caller, LPCSTR szArgs)
{
	char token[MAX_LINE_LENGTH], szExtra[MAX_LINE_LENGTH];
	char* tokens[1];
	tokens[0] = token;
	LCTokenize(szArgs, tokens, 1, szExtra);
	CGroup *pGroup = GetGroupByName(tokens[0]);
	if (!pGroup)
	{
		BangFailed("SetBackgroundColor", token);
		return;
	}
	pGroup->SetBackgroundColor(utils::String2Color(szExtra));
}


/*void bangHook	(HWND caller, LPCSTR szArgs)
{
	char token[MAX_LINE_LENGTH], extra[MAX_LINE_LENGTH];
	char* tokens[1];
	tokens[0] = token;
	LCTokenize(szArgs, tokens, 1, extra);
	CGroup *pGroup = GetGroupByName(tokens[0]);  <-- Here must be tokens[0] ;)
	if (!pGroup)
	{
		BangFailed("Hook", tokens[0]);  <-- Here must be tokens[0] ;)
		return;
	}

	char *handle = strrchr(szArgs, ' ');
	if (handle) 
	{
		HWND hBoxWnd = (HWND)atoi(handle+1);
		pGroup->Hook(hBoxWnd);
	}
}*/

void bangToggleExplorerDesktop	(HWND caller, LPCSTR szArgs)
{
	CGroup *pGroup = GetGroupByName(szArgs);
	if (pGroup)
		pGroup->ToggleExplorerDesktop();
	else
		BangFailed("ToggleExplorerDesktop", szArgs);
}